!  Simple program demonstrating use of the gdxf90 library.

program demo
use gdxf90
implicit none
integer :: k,l

!	Set element identifiers are 32 characters in length:

character (len=32), allocatable :: i(:), j(:)

!	Data may be returned in dense format:

real (kind=8) :: s
real (kind=8), allocatable :: v(:), m(:,:)

!	Alternatively, data may be accessed in sparse format:

real (kind=8), allocatable :: md(:)
integer, allocatable :: me(:,:)

!	Read the data file through the GDXREAD routine:

if (.not.gdxread('demo.gdx'))	stop 'Error reading demo.gdx.'

!	Read single-dimensional sets and parameters through the 
!	GDXDATA routine.  

if (.not.gdxdata(i,'i'))		stop 'Error reading set i'
if (.not.gdxdata(j,'j'))		stop 'Error reading set j'
if (.not.gdxdata(s,'s'))		stop 'Error reading parameter s'
if (.not.gdxdata(v,'v(i)'))		stop 'Error reading parameter v(i)'
if (.not.gdxdata(m,'m(i,j)'))		stop 'Error reading parameter m(i,j) (dense)'
if (.not.gdxdata(md,me,'m(i,j)'))	stop 'Error reading parameter m(i,j) (sparse)'

!	Produce an echo-print of data to the console:

write(*,'(//a)') 'SET i:';	do k=1,size(i);  write(*,'(a)') trim(i(k)); end do
write(*,'(//a)') 'SET j:';	do k=1,size(j);  write(*,'(a)') trim(j(k)); end do
write(*,'(//a)') 'SCALAR s:';	write(*,*) s
write(*,'(//a)') 'PARAMETER v:';  do k=1,size(v);  write(*,*) v(k); end do
write(*,'(//a)') 'PARAMETER m(i,j): (dense format)'
do k=1,size(m,1);  write(*,'(10f8.2)') (m(k,l),l=1,size(m,2)); end do
write(*,'(//a)') 'PARAMETER m(i,j): (sparse format)'
do k=1,size(md);  write(*,*) me(k,1),me(k,2), md(k); end do;

end program demo
