program gdx2dat
use gdxf90

	character (len=255) :: cmdlin, fname, item, ident, arglist=' ', suffix, itemlist(20)
	integer,	    allocatable :: e(:,:)
	real (kind=8),	    allocatable :: d(:)
	character (len=32), allocatable :: sarray(:)
	character (len=80), allocatable :: tarray(:)

	logical :: readsets = .false., readok
	type (gdxfile), pointer :: f
	type (variable), pointer :: v
	type (parameter), pointer :: p
	type (set), pointer :: s

	call getcmds(fname,readsets,itemlist)

	if (readsets) write(*,'(a)') '* Reading SETS from: '//trim(fname)//'.set'

	readok = gdxread(trim(fname)//'.gdx')
	if (.not.readok) stop 'Error reading GDX file.'
	f => filelist%next
	if (readsets) call inputsets(trim(fname)//'.set')

	if (itemlist(1).eq.' ') then
	  s => f%sets
	  do while (associated(s%next))
	    s => s%next
	    if (s%ndim.eq.1) then
	      call processset(s%ident)
	    else 
	      call processtuple(s%ident)
	      do k=1,s%ndim
	        write(item,'(50a)') trim(s%ident)//'(',(',',j=1,k-1),'.',(',',j=k,s%ndim-1),')'
		call processset(item)
	      end do
	    end if
	  end do

	  p => f%parameters
	  do while (associated(p%next))
	    p => p%next
	    call processparam(p%ident)
	    if (p%ndim.ge.1) then
	      do k=1,p%ndim
	        write(item,'(50a)') trim(p%ident)//'(',(',',j=1,k-1),'.',(',',j=k,p%ndim-1),')'
		call processset(item)
	      end do
	    end if
	  end do

	  v => f%variables
	  do while (associated(v%next))
	    v => v%next
	    call processvar(trim(v%ident)//'.l')
	    if (v%ndim.ge.1) then
	      do k=1,v%ndim
	        write(item,'(50a)') trim(v%ident)//'(',(',',j=1,k-1),'.',(',',j=k,v%ndim-1),')'
		call processset(item)
	      end do
	    end if
	  end do
	end if

	do k=1,20
	  if (itemlist(k).eq.' ') stop
	  item = itemlist(k);

	  ident = item; 
	  i = index(ident,'('); 
	  if (i.gt.0) then
	    arglist = ident(i:)
	    ident(i:) = ' '
	  else
	    arglist = ' '
	  endif
	  
	  i = index(ident,'.')
	  if (i.gt.0) then
	    suffix = ident(i:)
	    ident(i:) = ' '
	  else
	    suffix = ' '
	  endif

	  if (param() .and. index(arglist,'.').eq.0) then
	    call processparam(item)
	  elseif (varble() .and. index(arglist,'.').eq.0) then
	    call processvar(item)
	  elseif (tuple() .and. index(arglist,'.').eq.0) then
	    call processtuple(item)
	  else
	    call processset(item)
	  end if
	end do

contains

subroutine processparam(item)
	character (len=*) :: item
	readok = gdxdata(d,e,item)
	write(*,'(//a)') 'PARAMETER '//trim(item)
	if (size(e,2).ne.1) then
	  write(*,'(i0,a)') size(e,2),' dimensions'
	else
	  write(*,'(i0,a)') size(e,2),' dimension'
	end if
	write(*,'(i0,a)') size(d),' nonzeros'
	do i=1,size(d)
	  write(*,*) (e(i,j),j=1,size(e,2)), d(i)
	end do
	deallocate(d,e)
	return
end subroutine processparam

subroutine processvar(item)
	character (len=*) :: item

	readok = gdxdata(d,e,item)
	write(*,'(//a)') 'VARIABLE '//trim(item)
	if (size(e,2).ne.1) then
	  write(*,'(i0,a)') size(e,2),' dimensions'
	else
	  write(*,'(i0,a)') size(e,2),' dimension'
	end if
	write(*,'(i0,a)') size(d),' nonzeros'
	do i=1,size(d)
	  write(*,*) (e(i,j),j=1,size(e,2)), d(i)
	end do
	deallocate(d,e)
	return
end subroutine processvar

subroutine processset(item)
	character (len=*) :: item	

	if (allocated(tarray)) deallocate(tarray)
	if (index(item,'(').eq.0) then
	  readok = gdxdata(sarray,tarray,item)
	else
	  readok = gdxdata(sarray,item)
	endif
	write(*,'(//a)') 'SET '//trim(item)
	write(*,'(i0,a)') size(sarray),' elements'
	do i=1,size(sarray)
	  if (allocated(tarray)) then
	    write(*,'(i0,a)') i,' '//trim(sarray(i))//' '//trim(tarray(i))
	  else
	    write(*,'(i0,a)') i,' '//trim(sarray(i))
	  end if
	end do
	deallocate(sarray)
	if (allocated(tarray)) deallocate(tarray)
	return
end subroutine processset

subroutine processtuple(item)
	character (len=*) :: item	

	readok = gdxdata(e,tarray,item)
	write(*,'(//a)') 'TUPLE '//trim(item)
	if (size(e,2).ne.1) then
	  write(*,'(i0,a)') size(e,2),' dimensions'
	else
	  write(*,'(i0,a)') size(e,2),' dimension'
	end if
	if (size(e,1).ne.1) then
	  write(*,'(i0,a)') size(e,1),' elements'
	else
	  write(*,'(i0,a)') size(e,1),' element'
	end if
	do i=1,size(e,1)
	  write(*,*) (e(i,j),j=1,size(e,2)), trim(tarray(i))
	end do
	deallocate(e)
	return
end subroutine processtuple

logical function param()
	param = .true.
	p => f%parameters
	do while (associated(p%next))
	  p => p%next
	  if (eqv(ident,p%ident)) return
	end do
	param = .false.
	return
	end function param

logical function varble()
	varble = .true.
	v => f%variables
	do while (associated(v%next))
	  v => v%next
	  if (eqv(ident,v%ident)) return
	end do
	varble = .false.
	return
	end function varble

logical function tuple()
	tuple = .true.
	s => f%sets
	do while (associated(s%next))
	  s => s%next
	  if (eqv(ident,s%ident)) then
	    if (s%ndim.eq.1 .and. arglist.eq.' ') tuple = .false.
	    return
	  end if
	end do
	tuple = .false.
	return
	end function tuple

subroutine inputsets(setfname)
	implicit none
	character(len=*) setfname
	character (len=255) :: record
	character (len=32), allocatable :: sarray(:)
	character (len=32) :: ident
	integer :: nele,i,j

	open(10,file=setfname)
	do 
	  read(10,'(a)',end=100) record
	  record = adjustl(record)
	  if (eqv(record(1:4),'set ')) then
	    ident = adjustl(record(5:))
	    read(10,*) nele
	    allocate(sarray(nele))
	    do i=1,nele
	      read(10,*) j, sarray(j)
	    end do
	    if (.not.defineset(sarray,ident)) stop "Error writing a user set."
	    deallocate(sarray)
	  end if
	end do
100	close(10)
	return
	end subroutine inputsets

	subroutine getcmds(fname,readsets,itemlist)
	use service_routines
	character (len=*) :: fname
	logical :: readsets
	character (len=255) :: itemlist(20)
	integer :: i, nitem
	character(len=255) cmd

	call getcl(cmd)
	if (index(cmd,'"').gt.0) then
	  write(*,'(a)') 'Error: double quotes are not permitted on the command line.  Use single quotes.'
	  stop
	end if

!	File name is first:

	call getarg(1,fname)

!	Read the rest:

	nitem = 0
	itemlist = ' '
	do i=2,20
	  call getarg(i,cmd)
	  if (cmd.eq.' ') return
	  if (eqv(cmd,'-s')) then
	    readsets = .true.
	  else
	    nitem = nitem + 1
	    itemlist(nitem) = cmd
	  end if
	end do
	return
	end subroutine getcmds

end program gdx2dat
